create or replace PACKAGE BODY  "FNOD_DATA"."SAMPLE_PKG" as

  g_award_owner constant varchar2(32) := '~~AWARD~~';

  -- forward declare
  procedure get_next_sample(
    p_owner varchar2,
    p_assigned_to varchar2,
    p_last_sample_id in out number,
    p_case_id out number    
  );

  procedure pre_generate_sample(
    p_owner varchar2
  );

  procedure post_generate_sample(
    p_owner varchar2,
    p_qual_rows number,
    p_max_rows number,
    p_sample_percent number
  );

  procedure get_next_sample_sup(
    p_owner varchar2,
    p_assigned_to varchar2,
    p_last_sample_id in out number,
    p_case_id out number    
  );

--------------------------------------------------
-- pre_generate_sample
--------------------------------------------------
procedure pre_generate_sample(p_owner varchar2) is
begin
  delete from fnod_data.samples
  where owner = p_owner;

  delete from fnod_data.random_sample;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- post_generate_sample
--------------------------------------------------
procedure post_generate_sample(
  p_owner varchar2,
  p_qual_rows number,
  p_max_rows number,
  p_sample_percent number
)

is
  l_max_rows number;

begin

  l_max_rows := least(ceil(p_qual_rows * p_sample_percent/100), p_max_rows);

  insert into fnod_data.samples(owner, case_id)
  select *
  from (
    select p_owner as owner, c.case_id
    from case_link c
    where c.case_id in (
      select case_id
      from random_sample
    )
    order by dbms_random.value
  )
  where rownum <= l_max_rows;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- generate_case_sample
--------------------------------------------------
function generate_case_sample(
  p_owner varchar2,
  p_from_date timestamp with time zone,
  p_to_date timestamp with time zone,
  p_source_system_id number,
  p_case_type_id number,
  p_region_id number,
  p_regional_office_id number,
  p_max_rows number,
  p_sample_percent number
)
return integer

is
  l_qry varchar2(1000);

begin
  pre_generate_sample(p_owner);

  l_qry :=
    'insert into fnod_data.random_sample ' ||
    'select c.case_id ' ||
    'from fnod_data.case_link c ' ||
    'where c.locked_status = ''PENDING'' ' ||
    'and c.load_dt >= trunc(:1) ' ||
    'and c.load_dt < trunc(:2)+1 ';

    if (p_source_system_id is not null) then
    l_qry := l_qry ||
    'and c.source_system_id = ' || p_source_system_id || ' ';
    end if;

    if (p_case_type_id is not null) then
    l_qry := l_qry ||
    'and c.case_type_id = ' || p_case_type_id || ' ';
    end if;

    if (p_region_id is not null or p_regional_office_id is not null) then
    l_qry := l_qry ||
    'and exists (' ||
    '  select ''x'' ' ||
    '  from case_flag_app ' ||
    '  where case_id = c.case_id ';
    end if;

    if (p_region_id is not null) then
    l_qry := l_qry ||
    '  and region_id = ' || p_region_id || ' ';
    end if;

    if (p_regional_office_id is not null) then
    l_qry := l_qry ||
    '  and regional_office_id = ' || p_regional_office_id || ' ';
    end if;

    if (p_region_id is not null or p_regional_office_id is not null) then
    l_qry := l_qry ||
    ') ';
    end if;

  dbms_output.put_line(l_qry);
  execute immediate l_qry using p_from_date, p_to_date;

  post_generate_sample(p_owner, sql%rowcount, p_max_rows, p_sample_percent);

  return sql%rowcount;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- generate_fnod_sample
--------------------------------------------------
function generate_fnod_sample(
  p_owner varchar2,
  p_from_date timestamp with time zone,
  p_to_date timestamp with time zone,
  p_source_system_id number,
  p_case_type_id number,
  p_region_id number,
  p_regional_office_id number,
  p_max_rows number,
  p_sample_percent number,
  p_username varchar2
)
return integer

is
  l_qry varchar2(1000);
  l_max_rows number;

begin
  pre_generate_sample(p_owner);

  l_qry :=
    'insert into fnod_data.random_sample ' ||
    'select c.case_id ' ||
    'from case_link c ' ||
    'join fnod_record f on c.case_id = f.case_id ' ||
    'left outer join regional_office r on f.regional_office_id = r.regional_office_id ' ||
    'where c.locked_status = ''COMPLETE'' ' ||
    '  and f.username like '''||p_username||''''||
    '  and f.FNOD_CREATED_DT >= trunc(:1) ' ||
    '  and f.FNOD_CREATED_DT < trunc(:2)+1 ';

    if (p_source_system_id is not null) then
      l_qry := l_qry ||
        '  and c.source_system_id = ' || p_source_system_id || ' ';
    end if;

    if (p_case_type_id is not null) then
      l_qry := l_qry ||
        '   and c.case_type_id = ' || p_case_type_id || ' ';
    end if;

    if (p_region_id is not null) then
      l_qry := l_qry ||
        '   and r.region_id = ' || p_region_id || ' ';
    end if;

    if (p_regional_office_id is not null) then
      l_qry := l_qry ||
        '   and r.regional_office_id = ' || p_regional_office_id || ' ';
    end if;

  dbms_output.put_line(l_qry);

  execute immediate l_qry using p_from_date, p_to_date;

  post_generate_sample(p_owner, sql%rowcount, p_max_rows, p_sample_percent);

  return sql%rowcount;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- clear_sample
--------------------------------------------------
function clear_sample(
  p_owner varchar2
)
return integer
is
begin

  -- Protect award records
  if p_owner = g_Award_Owner then
    return 0;
  end if;

  delete from fnod_data.samples
  where owner = p_owner;

  return sql%rowcount;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- get_next
--------------------------------------------------
procedure get_next( p_username varchar2, p_last_sample_id in out number,p_case_id out number)
as
begin
  get_next_sample(p_username, p_username, p_last_sample_id, p_case_id);
exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);
end;

--------------------------------------------------
-- get_next_sample
--------------------------------------------------
procedure get_next_sample(p_owner varchar2, p_assigned_to varchar2, p_last_sample_id in out number, p_case_id out number)
as
  l_accept number := 0;
 begin

  if p_last_sample_id is null then
    p_last_sample_id := 0;
  end if;
  
  loop
  begin

    exit when l_accept > 3;
    
    update (  select sample_id, owner, case_id, assigned_to
              from fnod_data.samples
              where (sample_id)
                  in ( select sample_id
                        from (  select sample_id, assigned_to
                                  from fnod_data.samples
                                 where owner = p_owner
                                   and ( (assigned_to is null and sample_id > p_last_sample_id) 
                                         or assigned_to = p_assigned_to )                                   
                                order by nvl(assigned_to,'.') desc, sample_id
                             ) 
                     where rownum = 1 )
          )
       set assigned_to = p_assigned_to
    returning sample_id, case_id into p_last_sample_id, p_case_id;

    -- We have successfully gotten sample item, so return
    return;
    
  exception
    when others then
       l_accept := l_accept + 1;
  end;
  end loop;

 end;

--------------------------------------------------
-- add_award_case
--------------------------------------------------
function add_award_case(
  p_case_id number
)
return integer

is
  l_existing_rows number;

begin
  select count(*) into l_existing_rows
  from fnod_data.samples
  where owner = g_award_owner
  and case_id = p_case_id;

  if l_existing_rows <= 0 then
      insert into fnod_data.samples(owner, case_id, assigned_to)
      values(g_award_owner, p_case_id, null);
   end if;

  return sql%rowcount;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

function cancel_award_audit( p_sample_id number)
return integer
is
begin
  
  update fnod_data.samples
     set assigned_to = null
   where sample_id = p_sample_id;
    
  return sql%rowcount;
  
end;

--------------------------------------------------
-- get_next_award_case
--------------------------------------------------
procedure get_next_award_case(p_username varchar2, p_last_sample_id in out number, p_case_id out number)
is
begin

  get_next_sample_sup(g_award_owner, p_username, p_last_sample_id, p_case_id);

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- delete_sample
--------------------------------------------------
function delete_sample( p_case_id number, p_username varchar2 )
return integer
is
begin
  delete from fnod_data.samples
  where owner = p_username
    and case_id = p_case_id;

  return sql%rowcount;

exception
  when others then
    raise_application_error(-20001, 'ERROR, SQL: '|| SQLCODE ||' MESSAGE: '|| SQLERRM);

end;

--------------------------------------------------
-- get_next_sample_sup
--------------------------------------------------
procedure get_next_sample_sup(p_owner varchar2, p_assigned_to varchar2, p_last_sample_id in out number, p_case_id out number)
as
  l_accept number := 0;
 begin

  if p_last_sample_id is null then
    p_last_sample_id := 0;
  end if;
  
  loop
  begin

    exit when l_accept > 3;
    
    update (  select sample_id, owner, case_id, assigned_to
              from fnod_data.samples
              where (sample_id)
                  in ( select sample_id
                        from (  select sample_id, assigned_to
                                  from fnod_data.samples
                                 where owner = p_owner
                                and sample_id > p_last_sample_id                             
                                order by nvl(assigned_to,'.') desc, sample_id
                             ) 
                     where rownum = 1 )
          )
       set assigned_to = p_assigned_to
    returning sample_id, case_id into p_last_sample_id, p_case_id;

    -- We have successfully gotten sample item, so return
    return;
    
  exception
    when others then
       l_accept := l_accept + 1;
  end;
  end loop;

 end;

end;
